import sys

from tambo import Transport
from merfi import backends
from merfi import iso
import merfi
from merfi.decorators import catches


class Merfi(object):
    _help = """
merfi: A utility to find Release files and sign them with a given backend
signature engine like rpm-sign or gpg.

Version: %s

Global Options:
--log, --logging    Set the level of logging. Acceptable values:
                    debug, warning, error, critical
--check             Don't perform any actions, attempt to determine what would
                    happen and spit output as similar as possible

Sub Commands:
%s

    """

    mapper = {
        'rpm-sign': backends.rpm_sign.RpmSign,
        'gpg': backends.gpg.Gpg,
        'iso': iso.Iso
    }

    def __init__(self, argv=None, parse=True):
        if argv is None:
            argv = sys.argv
        if parse:
            self.main(argv)

    def help(self):
        sub_help = '\n'.join(['%-19s %s' % (
            sub.name, getattr(sub, 'help_menu', ''))
            for sub in self.mapper.values()])
        return self._help % (merfi.__version__, sub_help)

    @catches((RuntimeError, KeyboardInterrupt))
    def main(self, argv):
        options = [['--log', '--logging']]
        parser = Transport(argv, mapper=self.mapper,
                           options=options, check_help=False,
                           check_version=False)
        parser.parse_args()
        merfi.config['verbosity'] = parser.get('--log', 'info')
        merfi.config['check'] = parser.has('--check')
        parser.catch_help = self.help()
        parser.catch_version = merfi.__version__
        parser.mapper = self.mapper
        if len(argv) <= 1:
            return parser.print_help()
        parser.dispatch()
        parser.catches_help()
        parser.catches_version()



import os
from merfi.collector import RepoCollector
import merfi
from merfi import logger, util
from merfi.backends import base


class Gpg(base.BaseBackend):
    help_menu = 'gpg handler for signing files'
    _help = """
Signs files with gpg. Crawls a given path looking for 'Release' files (by
default)

Default behavior will perform these actions on 'Release' files::

    gpg --armor --detach-sig --output Release.gpg Release
    gpg --clearsign --output InRelease Release

%s

Options:

Positional Arguments:

[path]        The path to crawl for signing release files. Defaults to current
              working directory
"""
    executable = 'gpg'
    name = 'gpg'

    def sign(self):
        logger.info('Starting path collection, looking for files to sign')
        repos = RepoCollector(self.path)
        paths = repos.debian_release_files

        if paths:
            logger.info('%s matching paths found' % len(paths))
            # FIXME: this should spit the actual verified command
            logger.info('will sign with the following commands:')
            logger.info('gpg --batch --yes --armor --detach-sig --output Release.gpg Release')
            logger.info('gpg --batch --yes --clearsign --output InRelease Release')
        else:
            logger.warning('No paths found that matched')

        for path in paths:
            if merfi.config.get('check'):
                new_gpg_path = path.split('Release')[0]+'Release.gpg'
                new_in_path = path.split('Release')[0]+'InRelease'
                logger.info('[CHECKMODE] signing: %s' % path)
                logger.info('[CHECKMODE] signed: %s' % new_gpg_path)
                logger.info('[CHECKMODE] signed: %s' % new_in_path)
            else:
                os.chdir(os.path.dirname(path))
                detached = ['gpg', '--batch', '--yes', '--armor', '--detach-sig', '--output', 'Release.gpg', 'Release']
                clearsign = ['gpg', '--batch', '--yes', '--clearsign', '--output', 'InRelease', 'Release']
                logger.info('signing: %s' % path)
                util.run(detached)
                util.run(clearsign)



from merfi.backends import rpm_sign
from merfi.tests.util import CallRecorder
from filecmp import cmp
import os
import pytest
from tambo import Transport

@pytest.fixture
def rpmsign(request):
    backend = rpm_sign.RpmSign([])
    backend.detached = CallRecorder()
    backend.clear_sign = CallRecorder()
    return backend

class RpmSign(object):

    def setup(self):
        self.backend = rpm_sign.RpmSign([])
        self.backend.detached = CallRecorder()
        self.backend.clear_sign = CallRecorder()
        # fake command-line args
        self.argv = ['merfi', '--key', 'mykey']
        self.backend.parser = Transport(
            self.argv, options=self.backend.options
        )
        self.backend.parser.parse_args()

        # args to merfi.backends.rpm_sign.util's run()
        self.detached = [
            'rpm-sign', '--key', 'mykey', '--detachsign',
            'Release', '--output', 'Release.gpg']
        # args to merfi.backends.rpm_sign.util's run_output()
        self.clearsign = [
            'rpm-sign', '--key', 'mykey',
            '--clearsign', 'Release']

    def sign(self, directory):
        self.backend.path = directory
        self.backend.sign()


class TestRpmSign(RpmSign):

    def test_sign_no_files(self, tmpdir):
        self.sign(str(tmpdir))
        assert self.backend.detached.calls == []
        assert self.backend.clear_sign.calls == []

    def test_sign_two_files_detached_and_clearsign(self, repotree):
        self.sign(repotree)
        assert len(self.backend.detached.calls) == 2
        assert len(self.backend.clear_sign.calls) == 2

    def test_sign_two_files_path(self, repotree):
        self.sign(repotree)
        assert self.backend.detached.calls[0][0][0] == self.detached
        assert self.backend.detached.calls[1][0][0] == self.detached

    def test_sign_two_files_command(self, repotree):
        self.sign(repotree)
        assert self.backend.detached.calls[0][0][0] == self.detached
        assert self.backend.detached.calls[1][0][0] == self.detached


class TestRpmClearSign(RpmSign):

    def test_sign_two_files_command(self, repotree):
        self.sign(repotree)
        # first call, second argument (the command to run)
        assert self.backend.clear_sign.calls[0][0][1] == self.clearsign
        # second call, second argument (the command to run)
        assert self.backend.clear_sign.calls[1][0][1] == self.clearsign

    def test_sign_two_files_path(self, repotree):
        self.sign(repotree)
        first_path = self.backend.clear_sign.calls[0][0][0]
        second_path = self.backend.clear_sign.calls[1][0][0]
        assert first_path.endswith('/Release')
        assert second_path.endswith('/Release')

class TestRpmSignKeyfile(RpmSign):

    def test_keyfile_path(self, repotree, rpmsign, tmpdir):
        backend = rpmsign
        # fake keyfile
        keyfile = tmpdir.join('RPM-GPG-KEY-testing')
        keyfile.write('-----BEGIN PGP PUBLIC KEY BLOCK-----')
        # fake command-line args
        argv = ['merfi', '--key', 'mykey', '--keyfile', str(keyfile)]
        backend.parser = Transport(argv, options=backend.options)
        backend.parser.parse_args()
        backend.path = repotree
        backend.sign()

        release_key = os.path.join(repotree, 'release.asc')
        assert cmp(str(keyfile), release_key)

class TestRpmSignNat(RpmSign):

    def test_keyfile_path(self, repotree, rpmsign):
        backend = rpmsign
        # fake command-line args
        argv = ['merfi', '--key', 'mykey', '--nat']
        backend.parser = Transport(argv, options=backend.options)
        backend.parser.parse_args()
        backend.path = repotree
        backend.sign()

        clearsign = ['rpm-sign', '--nat', '--key', 'mykey', '--clearsign', 'Release']

        # first call, second argument (the command to run)
        assert backend.clear_sign.calls[0][0][1] == clearsign
        # second call, second argument (the command to run)
        assert backend.clear_sign.calls[1][0][1] == clearsign



import os
import shutil
from tambo import Transport
import merfi
from merfi import logger
from merfi import util
from merfi.collector import RepoCollector
from merfi.backends import base


class RpmSign(base.BaseBackend):
    help_menu = 'rpm-sign handler for signing files'
    _help = """
Signs files with rpm-sign. Crawls a given path looking for 'Release' files (by
default)

%s

Options

--key         Name of the key to use (see rpm-sign --list-keys)
--keyfile     Full path location of the public keyfile, for example
              /etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release
              or /etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-beta
--nat         A NAT is between this system and the signing server.

Positional Arguments:

[path]        The path to crawl for signing release files. Defaults to current
              working directory
    """
    executable = 'rpm-sign'
    name = 'rpm-sign'
    options = ['--key', '--keyfile', '--nat']

    def clear_sign(self, path, command):
        """
        When doing a "clearsign" with rpm-sign, the output goes to stdout, so
        that needs to be captured and written to the default output file for
        clear signed signatures (InRelease).
        """
        logger.info('signing: %s' % path)
        out, err, code = util.run_output(command)
        absolute_directory = os.path.dirname(os.path.abspath(path))
        with open(os.path.join(absolute_directory, 'InRelease'), 'w') as f:
            f.write(out)

    def detached(self, command):
        return util.run(command)

    def sign(self):
        self.keyfile = self.parser.get('--keyfile')
        if self.keyfile and not os.path.isfile(self.keyfile):
            raise RuntimeError('%s is not a file' % self.keyfile)
        self.key = self.parser.get('--key')
        if not self.key:
            raise RuntimeError('specify a --key for signing')
        logger.info('Starting path collection, looking for files to sign')
        repos = RepoCollector(self.path)
        paths = repos.debian_release_files

        if paths:
            logger.info('%s matching paths found' % len(paths))
            # FIXME: this should spit the actual verified command
            logger.info('will sign with the following commands:')
            logger.info('rpm-sign --key "%s" --detachsign Release --output Release.gpg' % self.key)
            logger.info('rpm-sign --key "%s" --clearsign Release --output InRelease' % self.key)
        else:
            logger.warning('No paths found that matched')

        for path in paths:
            if merfi.config.get('check'):
                new_gpg_path = path.split('Release')[0]+'Release.gpg'
                new_in_path = path.split('Release')[0]+'InRelease'
                logger.info('[CHECKMODE] signing: %s' % path)
                logger.info('[CHECKMODE] signed: %s' % new_gpg_path)
                logger.info('[CHECKMODE] signed: %s' % new_in_path)
            else:
                os.chdir(os.path.dirname(path))
                detached = ['rpm-sign', '--key', self.key, '--detachsign', 'Release', '--output', 'Release.gpg']
                clearsign = ['rpm-sign', '--key', self.key, '--clearsign', 'Release']
                if self.parser.has('--nat'):
                    detached.insert( 1, '--nat')
                    clearsign.insert( 1, '--nat')
                logger.info('signing: %s' % path)
                self.detached(detached)
                self.clear_sign(path, clearsign)

        if self.keyfile:
            logger.info('using keyfile "%s" as release.asc' % self.keyfile)
            for repo in repos:
                logger.info('placing release.asc in %s' % repo)
                if merfi.config.get('check'):
                    logger.info('[CHECKMODE] writing release.asc')
                else:
                    shutil.copyfile(
                        self.keyfile,
                        os.path.join(repo, 'release.asc'))



import os
import pytest
import subprocess
from merfi.iso import Iso
from merfi.util import which

class TestIso(object):

    def create_fake_iso(self, output_dir):
        """ Create a fake ISO file, without genisoimage """
        iso = Iso([])
        f = output_dir.join('test.iso')
        f.write('ISOCONTENTS')
        iso.output = str(f)
        iso.make_sha256sum()
        return iso

    def create_real_iso(self, output_dir):
        """ Create a "real" ISO file, using make_iso() (ie genisoimage) """
        # simple contents
        compose_dir = output_dir.mkdir('my-test-contents')
        f = compose_dir.join('contents.txt')
        f.write('This text file will be on our ISO')

        iso = Iso([])
        argv = ['merfi', str(output_dir.join('my-test-contents'))]
        iso.parse_args(argv)
        iso.make_iso()
        iso.make_sha256sum()
        return iso

    def test_sha256sum_contents(self, tmpdir):
        iso = self.create_fake_iso(tmpdir)
        with open(iso.output_checksum, 'r') as chsumf:
            assert chsumf.read() == "d8d322f6864229f8c9ef1b0845dd9e182c563c508fec30618fdb9b57c70a0147  test.iso\n"

    # Validate output_checksum's syntax with sha256sum.
    # The reason we shell out to sha256sum here is that it functionally
    # validates what a user would do with this ISO's checksum file: a user
    # would run `sha256sum -c` on it.
    @pytest.mark.skipif(which('sha256sum') is None, reason='sha256sum is not installed')
    def test_sha256sum_command(self, tmpdir):
        iso = self.create_fake_iso(tmpdir)
        os.chdir(os.path.dirname(iso.output_checksum))
        assert subprocess.call(['sha256sum', '-c', iso.output_checksum]) == 0

    @pytest.mark.skipif(which('genisoimage') is None, reason='genisoimage is not installed')
    def test_make_iso(self, tmpdir):
        iso = self.create_real_iso(tmpdir)
        assert os.path.isfile(str(tmpdir.join('my-test-contents-dvd.iso')))



"""
This logging facility was greatly inspired by Outlook 2003.
"""
import sys

CRITICAL = 5
ERROR = 4
WARNING = 3
INFO = 2
DEBUG = 1

_level_names = {
    CRITICAL : 'critical',
    WARNING  : 'warning',
    INFO     : 'info',
    ERROR    : 'error',
    DEBUG    : 'debug'
}

_reverse_level_names = dict((v, k) for (k, v) in _level_names.items())

_level_colors = {
    'remote'   : 'bold',
    'critical' : 'red',
    'warning'  : 'yellow',
    'info'     : 'blue',
    'debug'    : 'blue',
    'error'    : 'red'
}


class LogMessage(object):

    def __init__(self, level_name, message, writer=None, config_level=None):
        self.level_name = level_name
        self.message = message
        self.writer = writer or sys.stdout
        self.config_level = config_level or self.get_config_level()

    def skip(self):
        if self.level_int >= self.config_level:
            return False
        return True

    def header(self):
        from merfi.util import colorize
        colored = colorize.make(self.base_string)
        return getattr(colored, self.level_color)

    @property
    def base_string(self):
        if self.config_level < 2:
            return "--> [%s]" % self.level_name
        return "-->"

    @property
    def level_int(self):
        if self.level_name == 'remote':
            return 2
        return _reverse_level_names.get(self.level_name, 4)

    @property
    def level_color(self):
        return _level_colors.get(self.level_name, 'info')

    def line(self):
        msg = self.message.rstrip('\n')
        return "%s %s\n" % (self.header(), msg)

    def write(self):
        if not self.skip():
            self.writer.write(self.line())

    def get_config_level(self):
        import merfi
        level = merfi.config.get('verbosity', 'error')
        return _reverse_level_names.get(level, 4)


def error(message):
    return LogMessage('error', message).write()


def debug(message):
    return LogMessage('debug', message).write()


def info(message):
    return LogMessage('info', message).write()


def warning(message):
    return LogMessage('warning', message).write()


def critical(message):
    return LogMessage('critical', message).write()





class CallRecorder(object):
    """
    A very simple helper to record what calls it got
    """

    def __init__(self):
        self.calls = []

    def __call__(self, *args, **kwargs):
        self.calls.append([args, kwargs])



from __future__ import with_statement
import os
import re
import merfi


class RepoCollector(list):

    def __init__(self, path=None, config=None, _eager=True):
        config = config or merfi.config
        self.path = self._abspath(path or '.')
        # making it easier to test
        if _eager:
            self._collect()

    def _abspath(self, path):
        if not path.startswith('/'):
            return os.path.abspath(path)
        return path

    def _collect(self):
        if os.path.isfile(self.path):
            raise SystemExit('"%s" is a file. Please specify a directory.'
                % self.path)

        # Check whether our root (self.path) is itself a repo.
        if self.is_debian_repo(self.path):
            self.append(self.path)
            return

        # ... if not, walk the tree looking for repos in subdirs.

        # Local is faster
        walk = os.walk
        path = self.path

        for root, dirs, files in walk(path):
            if self.is_debian_repo(root):
                self.append(root)
                continue

    def is_debian_repo(self, directory):
        """ Is 'directory' a Debian repository ? """
        join = os.path.join
        isdir = os.path.isdir

        if not isdir(join(directory, 'dists')):
            return False
        if not isdir(join(directory, 'pool')):
            return False
        return True

    @property
    def debian_release_files(self):
        """ Find all the "Release" files to be signed within a Debian repo """
        result = []

        # Local is faster
        walk = os.walk
        join = os.path.join
        isfile = os.path.isfile

        for repo_path in self:
            for root, dirs, files in walk(repo_path):
                for dist in dirs:
                    release_file = join(root, dist, 'Release')
                    if isfile(release_file):
                        result.append(release_file)
        return result



import subprocess
import sys
from select import select
import os
from merfi import logger


def infer_path(arguments):
    """
    Infer the path from a list of potential paths. List might be empty and with
    items that are not paths.

    If arguments is an empty list it defaults to using the current working
    directory.
    """
    try:
        path = arguments[-1]
    except IndexError:
        # we didn't get an explicit path to work with so just return the
        # current working directory
        return os.getcwd()
    if os.path.exists(path):
        return os.path.abspath(path)
    else:
        raise RuntimeError('"%s" is not a valid path' % path)


def which(executable):
    """find the location of an executable"""
    if 'PATH' in os.environ:
        envpath = os.environ['PATH']
    else:
        envpath = os.defpath
    PATH = envpath.split(os.pathsep)

    locations = PATH + [
        '/usr/local/bin',
        '/bin',
        '/usr/bin',
        '/usr/local/sbin',
        '/usr/sbin',
        '/sbin',
    ]

    for location in locations:
        executable_path = os.path.join(location, executable)
        if os.path.isfile(executable_path) and os.access(executable_path, os.X_OK):
            return executable_path


def check_dependency(executable, silent=False):
    """
    Raise a RuntimeError if the dependency is not available in $PATH
    """
    if not which(executable):
        if not silent:
            logger.error('could not find %s' % executable)
        raise RuntimeError('%s needs to be installed and available in $PATH' % executable)


class colorize(str):
    """
    Pretty simple to use::

        colorize.make('foo').bold
        colorize.make('foo').green
        colorize.make('foo').yellow
        colorize.make('foo').red
        colorize.make('foo').blue

    Otherwise you could go the long way (for example if you are
    testing this class)::

        string = colorize('foo')
        string._set_attributes()
        string.red

    """

    def __init__(self, string):
        self.stdout = sys.__stdout__
        self.appends = ''
        self.prepends = ''
        self.isatty = self.stdout.isatty()

    def _set_attributes(self):
        """
        Sets the attributes here because the str class does not
        allow to pass in anything other than a string to the constructor
        so we can't really mess with the other attributes.
        """
        for k, v in self.__colors__.items():
            setattr(self, k, self.make_color(v))

    def make_color(self, color):
        if not self.isatty or self.is_windows:
            return self
        return color + self + '\033[0m' + self.appends

    @property
    def __colors__(self):
        return  dict(
                blue   = '\033[34m',
                green  = '\033[92m',
                yellow = '\033[33m',
                red    = '\033[91m',
                bold   = '\033[1m',
                ends   = '\033[0m'
        )

    @property
    def is_windows(self):
        if sys.platform == 'win32':
            return True
        return False

    @classmethod
    def make(cls, string):
        """
        A helper method to return itself and workaround the fact that
        the str object doesn't allow extra arguments passed in to the
        constructor
        """
        obj = cls(string)
        obj._set_attributes()
        return obj

#
# Common string manipulations
#
red_arrow = colorize.make('-->').red
blue_arrow = colorize.make('-->').blue


def run_output(command, **kw):
    logger.info('Running command: %s' % ' '.join(command))
    return _run_output(command, **kw)


def _run_output(cmd, verbose=False, **kw):
    process = subprocess.Popen(
        cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, **kw
    )
    stdout = [line.strip('\n') for line in process.stdout.readlines()]
    stderr = [line.strip('\n') for line in process.stderr.readlines()]
    if verbose:
        for line in stdout:
            logger.debug(line)
        for line in stderr:
            logger.warning(stderr)
    return '\n'.join(stdout), '\n'.join(stderr), process.wait()


def run(command, **kw):
    logger.info('Running command: %s' % ' '.join(command))
    _run(command, stop_on_nonzero=True, **kw)


def _run(cmd, **kw):
    stop_on_nonzero = kw.pop('stop_on_nonzero', True)

    process = subprocess.Popen(
        cmd,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        close_fds=True,
        **kw
    )

    while True:
        reads, _, _ = select(
            [process.stdout.fileno(), process.stderr.fileno()],
            [], []
        )

        for descriptor in reads:
            if descriptor == process.stdout.fileno():
                read = process.stdout.readline()
                if read:
                    logger.info(read)
                    sys.stdout.flush()

            if descriptor == process.stderr.fileno():
                read = process.stderr.readline()
                if read:
                    logger.warning(read)
                    sys.stderr.flush()

        if process.poll() is not None:
            while True:
                for descriptor in reads:
                    if descriptor == process.stdout.fileno():
                        read = process.stdout.readline()
                        if read:
                            logger.info(read)
                            sys.stdout.flush()

                    if descriptor == process.stderr.fileno():
                        read = process.stderr.readline()
                        if read:
                            logger.warning(read)
                            sys.stderr.flush()
                # At this point we have gone through all the possible
                # descriptors and `read` was empty, so we now can break out of
                # this since all stdout/stderr has been properly flushed to
                # logging
                if not read:
                    break

            break

    returncode = process.wait()
    if returncode != 0:
        if stop_on_nonzero:
            raise RuntimeError(
                "command returned non-zero exit status: %s" % returncode
            )
        else:
            logger.warning("command returned non-zero exit status: %s" % returncode)




import sys
from merfi.util import red_arrow
from functools import wraps


def catches(catch=None, handler=None, exit=True):
    """
    Very simple decorator that tries any of the exception(s) passed in as
    a single exception class or tuple (containing multiple ones) returning the
    exception message and optionally handling the problem if it rises with the
    handler if it is provided.

    So instead of douing something like this::

        def bar():
            try:
                some_call()
                print "Success!"
            except TypeError, exc:
                print "Error while handling some call: %s" % exc
                sys.exit(1)

    You would need to decorate it like this to have the same effect::

        @catches(TypeError)
        def bar():
            some_call()
            print "Success!"

    If multiple exceptions need to be catched they need to be provided as a
    tuple::

        @catches((TypeError, AttributeError))
        def bar():
            some_call()
            print "Success!"
    """
    catch = catch or Exception

    def decorate(f):

        @wraps(f)
        def newfunc(*a, **kw):
            try:
                return f(*a, **kw)
            except catch as e:
                if handler:
                    return handler(e)
                else:
                    sys.stderr.write(make_exception_message(e))
                    if exit:
                        sys.exit(1)
        return newfunc

    return decorate

#
# Decorator helpers
#


def make_exception_message(exc):
    """
    An exception is passed in and this function
    returns the proper string depending on the result
    so it is readable enough.
    """
    if str(exc):
        return '%s %s: %s\n' % (red_arrow, exc.__class__.__name__, exc)
    else:
        return '%s %s\n' % (red_arrow, exc.__class__.__name__)



from mock import call, patch
import pytest
from merfi.backends import gpg

class TestGpg(object):

    backend = gpg.Gpg([])

    # args to merfi.backends.gpg.util's run()
    detached = ['gpg', '--batch', '--yes', '--armor', '--detach-sig', '--output', 'Release.gpg', 'Release']
    clearsign = ['gpg', '--batch', '--yes', '--clearsign', '--output', 'InRelease', 'Release']

    @patch("merfi.backends.gpg.util")
    def test_sign_no_files(self, m_util, tmpdir):
        self.backend.path = str(tmpdir)
        self.backend.sign()
        assert not m_util.run.called

    @patch("merfi.backends.gpg.util")
    def test_sign_two_files(self, m_util, repotree):
        self.backend.path = repotree
        self.backend.sign()
        # Our repotree fixture has two "Release" files.
        # Each one gets detached-signed and clearsign'd.
        calls = [
            call(self.detached),
            call(self.clearsign),
            call(self.detached),
            call(self.clearsign),
        ]
        m_util.run.assert_has_calls(calls)



from hashlib import sha256
import os
from tambo import Transport
import merfi
from merfi import logger, util, base


class Iso(base.BaseCommand):
    help_menu = 'simple ISO manipulation'
    _help = """
A very simple interface to create ISOs with some sane defaults (like ensure
source directory has proper read permissions).

Default behavior will perform these actions on a source directory::

    genisoimage -r -o sourcedir-dvd.iso {source directory}
    sha256sum sourcedir-dvd.iso > sourcedir-dvd.iso.SHA256SUM

%s

Options:

-o, --output      Custom filename output (defaults to '<sourcedir>-dvd.iso').

Positional Arguments:

[source directory]  The path to create the ISO from. Required with no defaults.
"""
    executable = 'genisoimage'
    name = 'iso'

    def parse_args(self, argv=None):
        """ pass argv during testing """
        if argv is None:
            argv = self.argv
        options = [['--output', '-o']]
        parser = Transport(argv, options=options)
        parser.catch_help = self.help()
        parser.parse_args()
        self.source = util.infer_path(parser.unknown_commands)
        self.output = parser.get('--output', self.source + '-dvd.iso')
        self.check_dependency()
        self.make_iso()
        self.make_sha256sum()

    def make_iso(self):
        cmd = ['genisoimage', '-r', '-o', self.output, self.source]
        util.run(cmd)

    def make_sha256sum(self):
        chsum = sha256()
        self.output_checksum = self.output + '.SHA256SUM'
        with open(self.output, 'rb') as f:
            for chunk in iter(lambda: f.read(4096), b""):
                chsum.update(chunk)
        digest = chsum.hexdigest()
        with open(self.output_checksum, 'w') as chsumf:
            chsumf.write("%s  %s\n" % (digest, os.path.basename(self.output)))



from pytest import raises
from merfi import util


class TestInferPath(object):

    def test_no_last_argument(self):
        args = ['gpg', '--output', 'signed']
        with raises(RuntimeError) as error:
            util.infer_path(args)
        assert 'is not a valid path' in str(error.value)

    def test_last_argument(self):
        args = ['gpg', '--output', 'signed', '/']
        result = util.infer_path(args)
        assert result == '/'

    def test_no_arguments(self):
        # the parser engine pops the current command so we can
        # certainly end up with an empty argv list
        result = util.infer_path([])
        assert result.startswith('/')


class TestDependencyCheck(object):

    def test_silent_does_not_raise(self):
        result = util.check_dependency('ls', silent=True)
        assert result is None

    def test_silent_does_not_output(self, capsys):
        util.check_dependency('ls', silent=True)
        out, err = capsys.readouterr()
        assert out == ''
        assert err == ''

    def test_not_silent_prints_when_erroring(self, capsys):
        with raises(RuntimeError):
            util.check_dependency('ffffffffffffff')
        out, err = capsys.readouterr()
        assert 'could not find' in out



from merfi import base, util
from tambo import Transport


class BaseBackend(base.BaseCommand):

    options = []
    parser = None

    def parse_args(self):
        self.parser = Transport(self.argv, options=self.options)
        self.parser.catch_help = self.help()
        self.parser.parse_args()
        self.path = util.infer_path(self.parser.unknown_commands)
        self.check_dependency()
        self.sign()

    def sign(self):
        raise NotImplemented()



import os
import pytest
import shutil

# TODO: Enhance this fixture so that we distinguish between the "Release"
# files that need to be signed and the ones that do not.
#   - Create more bogus "Releases" files that don't need to be signed.
#   - Write a line "SHA256:" into the correct "Releases" file that *should* be
#     signed.
# See https://github.com/alfredodeza/merfi/issues/6
@pytest.fixture(scope="function")
def repotree(request, tmpdir):
    # Create a basic skeleton repository with "Release" files to sign.
    top_dir = str(tmpdir)
    # Top directories:
    os.mkdir(os.path.join(top_dir, 'db'))
    os.mkdir(os.path.join(top_dir, 'dists'))
    os.mkdir(os.path.join(top_dir, 'pool'))
    # Distro "Release" files:
    for distro in ['precise', 'trusty']:
        distro_dir = os.path.join(top_dir, 'dists', distro)
        os.mkdir(distro_dir)
        release_file = open(os.path.join(distro_dir, 'Release'), 'w')
        release_file.write('some Release metadata for %s' % distro)
        release_file.close()

    return top_dir


@pytest.fixture(scope="function")
def nested_repotree(request, tmpdir):
    # Create a basic skeleton repository with "Release" files to sign.
    directory = str(tmpdir)
    os.makedirs(os.path.join(directory, 'nested'))
    top_dir = os.path.join(directory, 'nested')
    # Top directories:
    os.mkdir(os.path.join(top_dir, 'db'))
    os.mkdir(os.path.join(top_dir, 'dists'))
    os.mkdir(os.path.join(top_dir, 'pool'))
    # Distro "Release" files:
    for distro in ['precise', 'trusty']:
        distro_dir = os.path.join(top_dir, 'dists', distro)
        os.mkdir(distro_dir)
        release_file = open(os.path.join(distro_dir, 'Release'), 'w')
        release_file.write('some Release metadata for %s' % distro)
        release_file.close()

    return top_dir




from merfi.collector import RepoCollector
from os.path import join, dirname


class TestRepoCollector(object):

    def setup(self):
        self.paths = RepoCollector(path='/', _eager=False)

    def test_simple_tree(self, repotree):
        paths = RepoCollector(path=repotree)
        # The root of the repotree fixture is itself a repository.
        expected = [ repotree ]
        assert set(paths) == set(expected)

    def test_path_is_absolute(self):
        assert self.paths._abspath('/') == '/'

    def test_path_is_not_absolute(self):
        assert self.paths._abspath('directory').startswith('/')

    def test_debian_release_files(self, repotree):
        paths = RepoCollector(repotree)
        release_files = paths.debian_release_files
        # The root of the repotree fixture is itself a repository.
        expected = [
            join(repotree, 'dists', 'trusty', 'Release'),
            join(repotree, 'dists', 'precise', 'Release'),
        ]
        assert set(release_files) == set(expected)

    def test_debian_nested_release_files(self, nested_repotree):
        # go one level up
        path = dirname(nested_repotree)
        paths = RepoCollector(path)
        release_files = paths.debian_release_files
        expected = [
            join(nested_repotree, 'dists', 'trusty', 'Release'),
            join(nested_repotree, 'dists', 'precise', 'Release'),
        ]
        assert set(release_files) == set(expected)



from merfi import base


class TestBase(object):

    def setup(self):
        self.base = base.BaseCommand(None)

    def test_dependency_help_is_installed(self):
        self.base.executable = 'ls'
        assert 'is installed and available' in self.base.dependency_help()

    def test_dependency_help_is_not_installed(self):
        self.base.executable = 'ffffffffffffffffffffff'
        assert 'is not installed or available' in self.base.dependency_help()



import os
from tambo import Transport
import merfi
from merfi import logger
from merfi import util


class BaseCommand(object):
    """
    A base backend class that holds common functionality for sub commands.
    """
    help_menu = 'help_menu attribute needs to be defined'
    _help = '_help class attribute needs to be defined'
    executable = 'executable class attribute needs to be defined'
    name = 'name class attribute needs to be defined'

    def __init__(self, argv):
        self.argv = argv

    def dependency_help(self):
        try:
            util.check_dependency(self.executable, silent=True)
            msg = '%s is installed and available in current $PATH'
            return util.colorize.make(msg).blue % self.executable
        except RuntimeError:
            msg = '%s is not installed or available in current $PATH'
            return util.colorize.make(msg).red % self.executable

    def check_dependency(self):
        util.check_dependency(self.executable)

    def help(self):
        return self._help % self.dependency_help()
